/*
udp-log-unix.c: send debug messages over udp
ed <irc.rizon.net>, 2020-04-09, MIT-Licensed

https://ocv.me/dev/?udp-log-unix.c  (linux/osx)
https://ocv.me/dev/?udp-log-win32.c (windows)
*/

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>

#ifdef __cplusplus
#include <cstdarg>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#else
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#endif


// sends  <sourcefile>  lineno  [function]
#define IM_HERE udp_msg("<%s> %4d @ [%s]\n", __FILE__, __LINE__, __FUNCTION__)

// sends  <sourcefile>  lineno  your-arguments
#define DBG(fmt, ...) udp_msg("<%s> %4d " fmt "\n", __FILE__, __LINE__, __VA_ARGS__)



void udp_msg(const char* fmt, ...) {
	const int port = 42829;
	const char* ip = "127.0.0.1";
	const int bufsz = 1024; //will realloc if necessary

	int fd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fd < 0)
		return;
	
	va_list args;
	char* buf = (char*)malloc(bufsz);
	
	va_start(args, fmt);
	ssize_t len = vsnprintf(buf, bufsz, fmt, args);
	va_end(args);
	
	if (len >= bufsz) {
		free(buf);
		buf = (char*)malloc(len);
		va_start(args, fmt);
		len = vsnprintf(buf, len, fmt, args);
		va_end(args);
	}
	
	struct sockaddr_in addr;
	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = inet_addr(ip);
	addr.sin_port = htons(port);
	
	sendto(fd, buf, len, 0, (struct sockaddr*)&addr, sizeof(addr));
	//fprintf(stderr, "# %s", buf);
	free(buf);
	close(fd);
}


int perftest() {
	for (int a = 0; a < 100 * 1000; a++)
		udp_msg("hello world fgsfds dsfargeg %d\n", a);

	return 0;
}


int main(int argc, char** argv) {
	//return perftest();

	IM_HERE;
	for (int a = 0; a < argc; a++)
		DBG("[%s]\n", argv[a]);
}


// receive messages:
//   socat -u udp4-recv:42829 -


// perftest results:
//  100k in 27.04 sec on 2017 macbook air (osx) ==   3698/sec
// 1000k in  4.17 sec on an E3-1246v3 (alpine)  == 239808/sec

